﻿using UnityEngine;
using System.Collections;

public delegate void CellChanged(MazeFieldCell.TileType type);

public struct Int2 {
	public int X, Y;
	public Int2(int x, int y) {
		X = x;
		Y = y;
	}
}

public class MazeFieldCell : MonoBehaviour {
		
	public MazeFieldCellViewController viewController;
	
	public event CellChanged OnCellChanged;
	public void CallCellChanged(MazeFieldCell.TileType type) 
	{
		if(OnCellChanged != null)
			OnCellChanged(type);
	}
	
	public enum TileType {
		None = 0,
		Brick = 1,
		Water = 2,
		Water_Up = 3,
		Water_Down = 4,
		Bridge = 5,
		Crater = 6,
		Bush = 7,
		Beton = 8,
		Base = 9
	}
	
	public enum ProjectileBehaviour {
		None,
		Destroyable,
		Immune
	}
	
	public enum PlayerBehaviour {
		Movable,
		Blocked
	}
	
	private TileType _tile;
	public TileType Tile 
	{
		get { return _tile; }
		set {
			_tile = value;
			
			switch(value) {
				
				case TileType.None: 
					ForPlayer = PlayerBehaviour.Movable;
					ForProjectile = ProjectileBehaviour.None;
					break;
				
				case TileType.Brick: 
					ForPlayer = PlayerBehaviour.Blocked;
					ForProjectile = ProjectileBehaviour.Destroyable;
					break;
				
				case TileType.Water: 
					ForPlayer = PlayerBehaviour.Blocked;
					ForProjectile = ProjectileBehaviour.None;
					break;
				
				case TileType.Water_Up: 
					ForPlayer = PlayerBehaviour.Blocked;
					ForProjectile = ProjectileBehaviour.None;
					break;
				
				case TileType.Water_Down: 
					ForPlayer = PlayerBehaviour.Blocked;
					ForProjectile = ProjectileBehaviour.None;
					break;
				
				case TileType.Bridge: 
					ForPlayer = PlayerBehaviour.Movable;
					ForProjectile = ProjectileBehaviour.None;
					break;
				
				case TileType.Crater: 
					ForPlayer = PlayerBehaviour.Movable;
					ForProjectile = ProjectileBehaviour.None;
					break;
				
				case TileType.Bush: 
					ForPlayer = PlayerBehaviour.Movable;
					ForProjectile = ProjectileBehaviour.None;
					break;
				
				case TileType.Beton: 
					ForPlayer = PlayerBehaviour.Blocked;
					ForProjectile = ProjectileBehaviour.Immune;
					break;
				
			case TileType.Base:
					ForPlayer = PlayerBehaviour.Blocked;
					ForProjectile = ProjectileBehaviour.Destroyable;
					break;
					
			}
		
			this.CallCellChanged(value);
		}
	}	
	
	public ProjectileBehaviour ForProjectile;
	public PlayerBehaviour ForPlayer;
	
	public int X = -1, Y = -1;
	
	public override string ToString ()
	{
		return X + ";" + Y;
	}
	
	public void Hit() {
		if(ForProjectile == ProjectileBehaviour.Destroyable) {
			Tile = TileType.Crater;
		}
	}
}
