﻿using UnityEngine;
using System.Collections;

public static class MazeFieldGenerator  
{	
	public static MazeField GenerateNewField(int FieldWidth, int FieldHeight) 
	{
		MazeField field = new MazeField();
		field.Cells = new MazeFieldCell[FieldWidth][];
		
		bool river = Random.Range(0, 1) == 1 ? true : false;
		
		for(int x = 0; x < FieldWidth; x++) 
		{
			field.Cells[x] = new MazeFieldCell[FieldHeight];
		}
		
		field.Cells[0][1] = GenerateCell(MazeFieldCell.TileType.None, 0, 1);
		field.Cells[0][5] = GenerateCell(MazeFieldCell.TileType.None, 0, 5);
		field.Cells[0][9] = GenerateCell(MazeFieldCell.TileType.None, 0, 9);
		
		// Player
		field.Cells[3][FieldWidth - 1] = GenerateCell(MazeFieldCell.TileType.None, 3, FieldWidth - 1);
		field.Cells[3][FieldWidth - 2] = GenerateCell(MazeFieldCell.TileType.None, 3, FieldWidth - 2);
		field.Cells[3][FieldWidth - 3] = GenerateCell(MazeFieldCell.TileType.None, 3, FieldWidth - 3);
		
		//Base
		field.Cells[4][FieldWidth - 1] = GenerateCell(MazeFieldCell.TileType.Brick, 4, FieldWidth - 1);
		field.Cells[5][FieldWidth - 1] = GenerateCell(MazeFieldCell.TileType.Base, 5, FieldWidth - 1);
		field.Cells[6][FieldWidth - 1] = GenerateCell(MazeFieldCell.TileType.Brick, 6, FieldWidth - 1);
		
		field.Cells[4][FieldWidth - 2] = GenerateCell(MazeFieldCell.TileType.Brick, 4, FieldWidth - 2);
		field.Cells[5][FieldWidth - 2] = GenerateCell(MazeFieldCell.TileType.Brick, 5, FieldWidth - 2);
		field.Cells[6][FieldWidth - 2] = GenerateCell(MazeFieldCell.TileType.Brick, 6, FieldWidth - 2);
		
		// REKA
		int riverWidth = Random.Range(2, 4);
		int startOfRiver = Random.Range(2, FieldHeight - riverWidth - 1);
		
		for(int x = 0; x < FieldWidth; x++) {
			for(int y = startOfRiver; y < startOfRiver + riverWidth; y++) {
				if(y == startOfRiver)
					field.Cells[x][y] = GenerateCell(MazeFieldCell.TileType.Water_Up, x, y);
				else if(y == startOfRiver + riverWidth - 1)
					field.Cells[x][y] = GenerateCell(MazeFieldCell.TileType.Water_Down, x, y);
				else field.Cells[x][y] = GenerateCell(MazeFieldCell.TileType.Water, x, y);
			}
		}
		
		// MOST
		int bridgeCount = Random.Range(2, 4);
		int bridgesSpace = FieldWidth / bridgeCount;
		
		for(int x = 0; x < bridgeCount; x++) {
			int bridgeX = x * bridgesSpace;
			for(int y = startOfRiver; y < startOfRiver + riverWidth; y++) {
				field.Cells[bridgeX][y] = GenerateCell(MazeFieldCell.TileType.Bridge, bridgeX, y);
			}	
		}	
		
		// BRICKS
		int bricksCount = Random.Range(10, 15);
		for(int i = 0; i < bricksCount; i++) {
			MazeFieldCell cell = null;
			
			int randX = Random.Range(0, FieldWidth);
			
			while(cell == null) {
				int randY = Random.Range(0, FieldHeight);
				if(field.Cells[randX][randY] == null) 
				{
					cell = GenerateCell(MazeFieldCell.TileType.Brick, randX, randY);
					field.Cells[randX][randY] = cell;
				}
			}
		}
		
		// BLOCKS
		int blocksCount = Random.Range(5, 10);
		for(int i = 0; i < blocksCount; i++) {
			MazeFieldCell cell = null;
			
			int randX = Random.Range(0, FieldWidth);
			
			while(cell == null) {
				int randY = Random.Range(0, FieldHeight);
				if(field.Cells[randX][randY] == null) 
				{
					cell = GenerateCell(MazeFieldCell.TileType.Beton, randX, randY);
					field.Cells[randX][randY] = cell;
				}
			}
		}
		
		// Kustiki
		int treesCount = Random.Range(5, 10);
		for(int i = 0; i < blocksCount; i++) {
			MazeFieldCell cell = null;
			
			int randX = Random.Range(0, FieldWidth);
			
			while(cell == null) {
				int randY = Random.Range(0, FieldHeight);
				if(field.Cells[randX][randY] == null) 
				{
					cell = GenerateCell(MazeFieldCell.TileType.Bush, randX, randY);
					field.Cells[randX][randY] = cell;
				}
			}
		}
		
		for(int x = 0; x < FieldWidth; x++) 
		{
			for(int y = 0; y < FieldHeight; y++) 
			{
				if(field.Cells[x][y] == null)
					field.Cells[x][y] = GenerateCell(MazeFieldCell.TileType.None, x, y);
			}
		}
		
//		MazeFieldCell.TileType type = (x % 2 == 0? MazeFieldCell.TileType.None : MazeFieldCell.TileType.Brick);
//				if(y % 2 == 0) type = MazeFieldCell.TileType.None;	
//					//(MazeFieldCell.TileType) Random.Range(0, 7);
//				
//				field.Cells[x][y] = GenerateCell(type, x, y);
		
		field.Width = FieldWidth;
		field.Height = FieldHeight;
		
		return field;
	}
			
	public static MazeFieldCell GenerateCell(MazeFieldCell.TileType tile, int x, int y) 
	{
		Transform cell = (GameObject.Instantiate(Resources.Load("Prefabs/Cell")) as GameObject).transform;
		
		var fieldCell = cell.gameObject.GetComponent<MazeFieldCell>();
		fieldCell.Tile = tile;
		fieldCell.X = x;
		fieldCell.Y = y;
		
		cell.parent = MazeGameViewController.sharedInstance.Background.transform.parent;
		cell.localScale = new Vector3(24, 24, 1);
		cell.localPosition = MazeField.GetCellCoords(x,y);
				
		return fieldCell;		
	}
	
//	public static MazeField GenerateNewField(int FieldWidth, int FieldHeight) 
//	{
//		MazeField field = new MazeField();
//		field.Cells = new MazeFieldCell[FieldWidth][];
//		
//		for(int x = 0; x < FieldWidth; x++) 
//		{
//			field.Cells[x] = new MazeFieldCell[FieldHeight];
//			
//			for(int y = 0; y < FieldHeight; y++) 
//			{
//				MazeFieldCell.TileType type = (x % 2 == 0? MazeFieldCell.TileType.None : MazeFieldCell.TileType.Brick);
//				if(y % 2 == 0) type = MazeFieldCell.TileType.None;	
//					//(MazeFieldCell.TileType) Random.Range(0, 7);
//				
//				field.Cells[x][y] = GenerateCell(type, x, y);
//			}
//		}
//		
//		
//		field.Width = FieldWidth;
//		field.Height = FieldHeight;
//		
//		return field;
//	}
}
