﻿using UnityEngine;
using System.Collections;


public class TankController : MonoBehaviour 
{
	public TankViewController viewController;
	
	public enum Side {
		Player, Enemy
	}
	
	public enum DirectionEnum {
		None,
		Up,
		Right, 
		Down, 
		Left
	}
	
	public float Speed = 24 * 2;
	
	private float DistanceToChangeDir = 4;
	
	
	public DirectionEnum _direction = DirectionEnum.Up;
	
	public DirectionEnum Direction {
		get { return _direction; }
		set { _direction = value; 
			CallDirectionChanged(value);
		}
	}
	
	[HideInInspector]
	public Int2 atCell { get; set; }
	
	public delegate void CellCenterReached(int x, int y);
	public delegate void DirectionChanged(DirectionEnum direction);
	
	public event CellCenterReached OnCellCenterReached;
	public void CallCellCenterReached(int x, int y) {
		if(OnCellCenterReached != null)
			OnCellCenterReached(x,y);
	}
	
	public event DirectionChanged OnDirectionChanged;
	public void CallDirectionChanged(DirectionEnum direction) {
		if(OnDirectionChanged != null)
			OnDirectionChanged(direction);
	}
	
	private Transform Projectile;
	public GameObject ProjectilePrefab;
		
		
	// Use this for initialization
	public void Move(float dirX, float dirY) 
	{
		int directionX = (int)dirX;
		int directionY = (int)-dirY;
		
		if(dirX == 0 && dirY == 0) return;		
		
		if(Vector2.Distance(MazeField.GetCellCoords(atCell.X, atCell.Y), transform.localPosition) < DistanceToChangeDir) {
			DirectionEnum newDir;
			
			if(Mathf.Abs(dirX) > Mathf.Abs(dirY)) {
				newDir = dirX > 0 ? DirectionEnum.Right : DirectionEnum.Left;
				directionY = 0;
			}			
			else {
				newDir = dirY > 0 ? DirectionEnum.Up : DirectionEnum.Down;
				directionX = 0;
			}
			
			if(newDir != Direction) Direction = newDir;
		}				
		
		// Ne mogem dvigatsya d dr napravlenii esli pokinuli center
		if(Mathf.Abs(directionY) > Mathf.Abs(directionX) && (Direction == DirectionEnum.Right || Direction == DirectionEnum.Left)) return;
		if(Mathf.Abs(directionX) > Mathf.Abs(directionY) && (Direction == DirectionEnum.Up || Direction == DirectionEnum.Down)) return;
		
		//Debug.Log(directionX + " - " + directionY);
		var cell = GetCellAtDirectionFromCurrent(atCell, directionX, directionY);
		
		if(cell == null) return;
		
		var cellCoords = MazeField.GetCellCoords(cell.X, cell.Y);
		
		if(CanMoveTo(cell)) {
			//Debug.Log("CanMove From: " + atCell.X + ":" + atCell.Y +  " To " + cell);
			transform.localPosition += new Vector3(directionX, -directionY, 0) * Speed * Time.fixedDeltaTime;
			
			
			// virovnyat
			if(Mathf.Abs(directionY) > Mathf.Abs(directionX)) transform.localPosition = new Vector3(cellCoords.x, transform.localPosition.y, transform.localPosition.z);
			else transform.localPosition = new Vector3(transform.localPosition.x, cellCoords.y, transform.localPosition.z);
			
			
			if(Vector2.Distance(cellCoords, transform.localPosition) < DistanceToChangeDir)
			{
				//Debug.Log("new " + cell.X + " : " + cell.Y);
				//transform.localPosition = cellCoords;
				
				atCell = new Int2(cell.X, cell.Y);
				CallCellCenterReached(atCell.X, atCell.Y);
			}
		}
		//else	Debug.Log("CANNOT Move From: " + atCell.X + ":" + atCell.Y +  " To " + cell);
	}
	
	// -1 ... 1
	public MazeFieldCell GetCellAtDirectionFromCurrent(Int2 cell, int directionX, int directionY) {
		
		// Esli mi dalshe centra svoei
		if(Vector2.Distance(MazeField.GetCellCoords(atCell.X, atCell.Y), transform.localPosition) > DistanceToChangeDir)
		{
			var atCellCoords = MazeField.GetCellCoords(atCell.X, atCell.Y);
			
				// Dvigaemsya k nei
			if(transform.localPosition.y < atCellCoords.y && directionY < 0) return MazeGameController.Field.Cells[atCell.X][atCell.Y];
			if(transform.localPosition.y > atCellCoords.y && directionY > 0) return MazeGameController.Field.Cells[atCell.X][atCell.Y];
			
			if(transform.localPosition.x > atCellCoords.x && directionX < 0) return MazeGameController.Field.Cells[atCell.X][atCell.Y];
			if(transform.localPosition.x < atCellCoords.x && directionX > 0) return MazeGameController.Field.Cells[atCell.X][atCell.Y];
		}
		
		bool canMoveHor = false, canMoveVer = false;
		
		if(directionX != 0)
			canMoveHor = (cell.X + directionX >= 0 && cell.X + directionX < MazeGameController.Field.Width);
		
		if(directionY != 0)
			canMoveVer = (cell.Y + directionY >= 0 && cell.Y + directionY < MazeGameController.Field.Height);
		
		if(!canMoveHor && !canMoveVer) return null;
		
		//if(canMoveHor) Debug.Log("Hor");
		//if(canMoveVer) Debug.Log("ver");
		
		if(Mathf.Abs(directionX) > Mathf.Abs(directionY) && canMoveHor) {
			return MazeGameController.Field.Cells[cell.X + directionX][cell.Y];
		}
		
		if(Mathf.Abs(directionY) > Mathf.Abs(directionX) && canMoveVer) {
			return MazeGameController.Field.Cells[cell.X][cell.Y + directionY];
		}
		
		Debug.Log ("WTF");		
		return null;
	}
	
	public bool CanMoveTo(MazeFieldCell cell) 
	{
		if(cell == null) return false;
		
		return cell.ForPlayer == MazeFieldCell.PlayerBehaviour.Movable;	
	}
	
	public void Shoot() {
		if(!Projectile) {
			Projectile = (GameObject.Instantiate(ProjectilePrefab) as GameObject).transform;
			
			Projectile.parent = MazeGameViewController.sharedInstance.root;
			Projectile.transform.localScale = Vector3.one * MazeGameViewController.sharedInstance.ProjectileSize;
			
			Vector3 dir = Vector3.zero;
			
			switch(Direction) {
				case DirectionEnum.Right:
					dir = transform.right;
					break;
				
				case DirectionEnum.Left:
					dir = -transform.right;
					break;
				
				case DirectionEnum.Up:
					dir = transform.up;
					break;
				
				case DirectionEnum.Down:
					dir = -transform.up;
					break;
			}
			
			Projectile.transform.localPosition = transform.localPosition + dir * MazeGameViewController.sharedInstance.ProjectileSize * 2;
			Projectile.transform.right = dir;
		}
	}
}
